@@ -6,15 +6,15 @@ static const char SCCSID[]="@(#)PJ_eqc.c	4.1	94/02/15	GIE	REL";
 #define PJ_LIB__
 # include	<projects.h>
 PROJ_HEAD(eqc, "Equidistant Cylindrical (Plate Caree)")
-	"\n\tCyl, Sph\n\tlat_ts=";
+	"\n\tCyl, Sph\n\tlat_ts=[, lat_0=0]";
 FORWARD(s_forward); /* spheroid */
 	xy.x = P->rc * lp.lam;
-	xy.y = lp.phi;
+	xy.y = lp.phi - P->phi0;
 	return (xy);
 }
 INVERSE(s_inverse); /* spheroid */
-	lp.phi = xy.y;
 	lp.lam = xy.x / P->rc;
+	lp.phi = xy.y + P->phi0;
 	return (lp);
 }
 FREEUP; if (P) pj_dalloc(P); }
