@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id$
+ * $Id: pj_gridlist.c,v 1.5 2006/11/17 22:16:30 mloskot Exp $
  *
  * Project:  PROJ.4
  * Purpose:  Code to manage the list of currently loaded (cached) PJ_GRIDINFOs
@@ -26,7 +26,25 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ ******************************************************************************
+ *
+ * $Log: pj_gridlist.c,v $
+ * Revision 1.5  2006/11/17 22:16:30  mloskot
+ * Uploaded PROJ.4 port for Windows CE.
+ *
+ * Revision 1.4  2005/11/01 05:56:13  fwarmerdam
+ * improved error handling if gridcount is zero
+ *
+ * Revision 1.3  2003/03/18 16:26:58  warmerda
+ * clear error if missing file is not required
+ *
+ * Revision 1.2  2003/03/17 19:45:47  warmerda
+ * support '@' marker for optional grids
+ *
+ * Revision 1.1  2003/03/15 06:01:18  warmerda
+ * New
+ *
+ */
 
 #define PJ_LIB__
 
@@ -46,6 +64,13 @@
 
 static PJ_GRIDINFO *grid_list = NULL;
 
+/* used only by pj_load_nadgrids() and pj_deallocate_grids() */
+
+static int           last_nadgrids_max = 0;
+static int           last_nadgrids_count = 0;
+static PJ_GRIDINFO **last_nadgrids_list = NULL;
+static char         *last_nadgrids = NULL;
+
 /************************************************************************/
 /*                        pj_deallocate_grids()                         */
 /*                                                                      */
@@ -61,7 +86,19 @@ void pj_deallocate_grids()
         grid_list = grid_list->next;
         item->next = NULL;
 
-        pj_gridinfo_free( pj_get_default_ctx(), item );
+        pj_gridinfo_free( item );
+    }
+
+    if( last_nadgrids != NULL )
+    {
+        pj_dalloc( last_nadgrids );
+        last_nadgrids = NULL;
+
+        pj_dalloc( last_nadgrids_list );
+        last_nadgrids_list = NULL;
+
+        last_nadgrids_count = 0;
+        last_nadgrids_max = 0;
     }
 }
 
@@ -72,14 +109,10 @@ void pj_deallocate_grids()
 /*      last_nadgrids_list.                                             */
 /************************************************************************/
 
-static int pj_gridlist_merge_gridfile( projCtx ctx, 
-                                       const char *gridname,
-                                       PJ_GRIDINFO ***p_gridlist,
-                                       int *p_gridcount, 
-                                       int *p_gridmax )
+static int pj_gridlist_merge_gridfile( const char *gridname )
 
 {
-    int got_match=0;
+    int i, got_match=0;
     PJ_GRIDINFO *this_grid, *tail = NULL;
 
 /* -------------------------------------------------------------------- */
@@ -98,26 +131,26 @@ static int pj_gridlist_merge_gridfile( projCtx ctx,
                 return 0;
 
             /* do we need to grow the list? */
-            if( *p_gridcount >= *p_gridmax - 2 )
+            if( last_nadgrids_count >= last_nadgrids_max - 2 )
             {
                 PJ_GRIDINFO **new_list;
-                int new_max = *p_gridmax + 20;
+                int new_max = last_nadgrids_max + 20;
 
                 new_list = (PJ_GRIDINFO **) pj_malloc(sizeof(void*) * new_max);
-                if( *p_gridlist != NULL )
+                if( last_nadgrids_list != NULL )
                 {
-                    memcpy( new_list, *p_gridlist,
-                            sizeof(void*) * (*p_gridmax) );
-                    pj_dalloc( *p_gridlist );
+                    memcpy( new_list, last_nadgrids_list, 
+                            sizeof(void*) * last_nadgrids_max );
+                    pj_dalloc( last_nadgrids_list );
                 }
 
-                *p_gridlist = new_list;
-                *p_gridmax = new_max;
+                last_nadgrids_list = new_list;
+                last_nadgrids_max = new_max;
             }
 
             /* add to the list */
-            (*p_gridlist)[(*p_gridcount)++] = this_grid;
-            (*p_gridlist)[*p_gridcount] = NULL;
+            last_nadgrids_list[last_nadgrids_count++] = this_grid;
+            last_nadgrids_list[last_nadgrids_count] = NULL;
         }
 
         tail = this_grid;
@@ -129,7 +162,7 @@ static int pj_gridlist_merge_gridfile( projCtx ctx,
 /* -------------------------------------------------------------------- */
 /*      Try to load the named grid.                                     */
 /* -------------------------------------------------------------------- */
-    this_grid = pj_gridinfo_init( ctx, gridname );
+    this_grid = pj_gridinfo_init( gridname );
 
     if( this_grid == NULL )
     {
@@ -146,8 +179,7 @@ static int pj_gridlist_merge_gridfile( projCtx ctx,
 /* -------------------------------------------------------------------- */
 /*      Recurse to add the grid now that it is loaded.                  */
 /* -------------------------------------------------------------------- */
-    return pj_gridlist_merge_gridfile( ctx, gridname, p_gridlist, 
-                                       p_gridcount, p_gridmax );
+    return pj_gridlist_merge_gridfile( gridname );
 }
 
 /************************************************************************/
@@ -160,18 +192,36 @@ static int pj_gridlist_merge_gridfile( projCtx ctx,
 /*      the cost of building the list of tables each time.              */
 /************************************************************************/
 
-PJ_GRIDINFO **pj_gridlist_from_nadgrids( projCtx ctx, const char *nadgrids, 
-                                         int *grid_count)
+PJ_GRIDINFO **pj_gridlist_from_nadgrids( const char *nadgrids, int *grid_count)
 
 {
     const char *s;
-    PJ_GRIDINFO **gridlist = NULL;
-    int grid_max = 0;
 
     pj_errno = 0;
     *grid_count = 0;
 
-    pj_acquire_lock();
+    if( last_nadgrids != NULL 
+        && strcmp(nadgrids,last_nadgrids) == 0 )
+    {
+        *grid_count = last_nadgrids_count;
+        if( *grid_count == 0 )
+            pj_errno = -38;
+
+        return last_nadgrids_list;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Free old one, if any, and make space for new list.              */
+/* -------------------------------------------------------------------- */
+    if( last_nadgrids != NULL )
+    {
+        pj_dalloc(last_nadgrids);
+    }
+    
+    last_nadgrids = (char *) pj_malloc(strlen(nadgrids)+1);
+    strcpy( last_nadgrids, nadgrids );
+
+    last_nadgrids_count = 0;
 
 /* -------------------------------------------------------------------- */
 /*      Loop processing names out of nadgrids one at a time.            */
@@ -192,10 +242,9 @@ PJ_GRIDINFO **pj_gridlist_from_nadgrids( projCtx ctx, const char *nadgrids,
              s[end_char] != '\0' && s[end_char] != ','; 
              end_char++ ) {}
 
-        if( end_char >= sizeof(name) )
+        if( end_char > sizeof(name) )
         {
-            pj_ctx_set_errno( ctx, -38 );
-            pj_release_lock();
+            pj_errno = -38;
             return NULL;
         }
         
@@ -206,19 +255,20 @@ PJ_GRIDINFO **pj_gridlist_from_nadgrids( projCtx ctx, const char *nadgrids,
         if( *s == ',' )
             s++;
 
-        if( !pj_gridlist_merge_gridfile( ctx, name, &gridlist, grid_count, 
-                                         &grid_max) 
-            && required )
+        if( !pj_gridlist_merge_gridfile( name ) && required )
         {
-            pj_ctx_set_errno( ctx, -38 );
-            pj_release_lock();
+            pj_errno = -38;
             return NULL;
         }
         else
             pj_errno = 0;
     }
 
-    pj_release_lock();
-
-    return gridlist;
+    if( last_nadgrids_count > 0 )
+    {
+        *grid_count = last_nadgrids_count;
+        return last_nadgrids_list;
+    }
+    else
+        return NULL;
 }
