@@ -2,7 +2,7 @@
 
 #include <projects.h>
 
-char const pj_release[]="Rel. 4.6.0, 21 Dec 2007";
+char const pj_release[]="Rel. 4.6.1, 21 August 2008";
 
 const char *pj_get_release()
 
